provider "aws" {
  alias   = "${var.account}"
  region  = "${var.region}"
  profile = "${var.account}"
}

# ALB
resource "aws_lb" "alb" {
  provider = "aws.${var.account}"
  name     = "${var.name}-${var.environment}"
  internal = "${var.internal}"

  subnets = [
    "${var.subnets}",
  ]

  security_groups = [
    "${var.security_groups}",
  ]

  tags {
    Name = "${var.name}-${var.environment}-alb"
  }
}

resource "aws_lb_target_group" "alb_target_group" {
  name     = "${var.name}-${var.environment}"
  port     = "${var.target_group_port}"
  protocol = "${var.protocol}"
  vpc_id   = "${var.vpc}"

  health_check {
    interval            = "${var.health_check_interval}"
    path                = "${var.health_check_path}"
    port                = "${var.health_check_port}"
    protocol            = "${var.protocol}"
    timeout             = "${var.health_check_timeout}"
    healthy_threshold   = "${var.healthy_threshold}"
    unhealthy_threshold = "${var.unhealthy_threshold}"
  }
}

resource "aws_lb_listener" "alb_listener" {
  load_balancer_arn = "${aws_lb.alb.arn}"
  port              = "443"
  protocol          = "HTTPS"
  ssl_policy        = "ELBSecurityPolicy-2016-08"
  certificate_arn   = "${var.certificate_arn}"

  default_action {
    target_group_arn = "${aws_lb_target_group.alb_target_group.arn}"
    type             = "forward"
  }
}

resource "aws_lb_listener" "alb_redirect_80" {
  load_balancer_arn = "${aws_lb.alb.arn}"
  port              = "80"
  protocol          = "HTTP"

  default_action {
    type = "redirect"

    redirect {
      port        = "443"
      protocol    = "HTTPS"
      status_code = "HTTP_301"
    }
  }
}

resource "aws_cloudwatch_metric_alarm" "alb_unhealthy" {
  alarm_name          = "${var.name}-${var.environment}-alb-unhealthy"
  comparison_operator = "GreaterThanThreshold"
  evaluation_periods  = "3"
  metric_name         = "UnHealthyHostCount"
  namespace           = "AWS/ApplicationELB"
  period              = "180"
  statistic           = "Maximum"
  threshold           = "0"
  alarm_actions       = ["${var.alert_arn}"]
  ok_actions          = ["${var.alert_arn}"]
  alarm_description   = "Notify pagerduty if any hosts are unhealthy for more than 9 minutes."

  dimensions = {
    LoadBalancer = "app/${aws_lb.alb.name}/${element(split("/", aws_lb.alb.arn), 3)}"
    TargetGroup  = "targetgroup/${aws_lb_target_group.alb_target_group.name}/${element(split("/", aws_lb_target_group.alb_target_group.arn), 2)}"
  }
}
